// Copyright (c) 2015, Microsoft Corporation
//
// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
// SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
// IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//

package com.microsoft.alljoyn.minecraft;

import java.io.File;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.StringTokenizer;

import org.alljoyn.bus.AboutListener;
import org.alljoyn.bus.AboutObjectDescription;
import org.alljoyn.bus.BusAttachment;
import org.alljoyn.bus.BusException;
import org.alljoyn.bus.Mutable;
import org.alljoyn.bus.ProxyBusObject;
import org.alljoyn.bus.SessionListener;
import org.alljoyn.bus.SessionOpts;
import org.alljoyn.bus.Status;
import org.alljoyn.bus.Variant;

public abstract class AllJoynClient {
	
	public static long MIN_UINT32 = 0L;
	public static long MAX_UINT32 = 4294967295L;
	public static long MIN_UINT16 = 0L;
	public static long MAX_UINT16 = 65535L;
	
    protected String advertisedName;
    protected String objPath;
    protected Class<?> busInterface;
    
    protected BusAttachment mBus;
    protected Map mProxyObjMap = new HashMap();
    
    static {
		// Load alljoyn_java
		try {
			System.loadLibrary("alljoyn_java");
		} catch (Throwable t) {
			// Try to load the DLL from PATH
			String paths = System.getenv("PATH");
			StringTokenizer parser = new StringTokenizer(paths, ";");
			while (parser.hasMoreTokens()) {
				String lib = parser.nextToken();
				if (lib.charAt(lib.length()-1) != '\\') {
					lib += "\\";
				}
				lib += "alljoyn_java.dll";
				File f = new File(lib);
				if(f.exists() && !f.isDirectory()) { 
					System.load(lib);
					break;
				}
			}
		}
    }
   
    public void setUInt32Property(final String property, final long value)
    {
		for (Object key: mProxyObjMap.keySet()) {
			final ProxyBusObject proxyObj = (ProxyBusObject)mProxyObjMap.get(key);
			Runnable task = new Runnable() {
		        @Override 
		        public void run() { 
		    		try {
		    			long pv = Math.min(MAX_UINT32, Math.max(value, MIN_UINT32));
		    			System.out.println("Set " + property + " to " + NumberFormat.getNumberInstance(Locale.US).format(pv));
		    			proxyObj.setProperty(busInterface, property, new Variant(pv, "u"));
		    		} catch (BusException ex) {
		    			ex.printStackTrace();
		    		}
		        }
			};
			new Thread(task, "ServiceThread_"+property). start();
		}
    }
    
    public void incrementUInt16Property(final String property, final long delta)
    {
    	if (delta == 0) {
    		return;
    	}
    	
		for (Object key: mProxyObjMap.keySet()) {
			final ProxyBusObject proxyObj = (ProxyBusObject)mProxyObjMap.get(key);
			Runnable task = new Runnable() {
		        @Override 
		        public void run() { 
		    		try {
		    			Variant v = proxyObj.getProperty(busInterface, property);
		    			long value = Integer.toUnsignedLong(v.getObject(short.class));
		    			long pv = Math.min(MAX_UINT16, Math.max(value+ delta, MIN_UINT16));
		    			System.out.println("Set " + property + " to " + NumberFormat.getNumberInstance(Locale.US).format(pv));
		    			proxyObj.setProperty(busInterface, property, new Variant(pv, "g"));
		    		} catch (BusException ex) {
		    			ex.printStackTrace();
		    		}
		        }
			};
			new Thread(task, "ServiceThread_"+property). start();
		}
    }
    
    public void initialize() {
		     
        class AJ_AboutListener implements AboutListener
        {
        	public void announced(String busName, int version, short port, AboutObjectDescription[] objectDescriptions, Map<String, Variant> aboutData) {
                System.out.println(String.format("Found service (%s, %d, %d)", busName, version, port));
                SessionOpts sessionOpts = new SessionOpts();
                sessionOpts.traffic = SessionOpts.TRAFFIC_MESSAGES;
                sessionOpts.isMultipoint = false;
                sessionOpts.proximity = SessionOpts.PROXIMITY_ANY;
                sessionOpts.transports = SessionOpts.TRANSPORT_ANY;

                Mutable.IntegerValue sessionId = new Mutable.IntegerValue();

                mBus.enableConcurrentCallbacks();

                Status status = mBus.joinSession(busName, port, sessionId, sessionOpts, new SessionListener(){
                	public void sessionLost(int lostSessionId){
                		Object obj = mProxyObjMap.remove(lostSessionId);
                		System.out.println(String.format("Session lost (%x), sessionId = %d", obj, lostSessionId));
                		
                		if (obj != null) {
	                        // Delete '.'s to the end of light text
	                        RenderEventHandler.drawDots((short)mProxyObjMap.size());
                		}
                	}
                });

                if (status != Status.OK) {
                    return;
                }
                LightChangeEventHandler.refreshNeeded = true;
                System.out.println(String.format("Joined session %d", sessionId.value));
                
                mProxyObjMap.put(sessionId.value, mBus.getProxyBusObject(busName, objPath, sessionId.value, new Class[] { busInterface }));
                
                // Append '.'s to the end of light text
                RenderEventHandler.drawDots((short)mProxyObjMap.size());
        	}
        }

       	mBus = new BusAttachment(getClass().getName(), BusAttachment.RemoteMessage.Receive);

        Status status = mBus.connect();
        if (status != Status.OK) {
            return;
        }
        
        AJ_AboutListener ajAboutListener = new AJ_AboutListener();
        mBus.registerAboutListener(ajAboutListener);
        
        status = mBus.whoImplements(new String[]{advertisedName});
        if (status != Status.OK) {
            return;
        }
        System.out.println("Finding service (" + advertisedName + ")...");
    }
}
