// Copyright (c) 2015, Microsoft Corporation
//
// Permission to use, copy, modify, and/or distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
// SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
// IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
//

package com.microsoft.alljoyn.minecraft;

import java.awt.Color;
import java.text.NumberFormat;
import java.util.Locale;

import net.minecraft.block.Block;
import net.minecraft.client.Minecraft;
import net.minecraft.client.entity.EntityPlayerSP;
import net.minecraft.client.multiplayer.WorldClient;
import net.minecraft.entity.Entity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.Vec3;

public class LightLevelHelper 
{
	public static float MAX_UINT32 = 4294967295.0F;
	private static float ZERO_BRIGHTNESS_THRESHOLD = 0.01F; 
	private static float BRIGHTNESS_SCALE = 0.8F;
	private static Vec3 TORCH_COLOR = new Vec3(0.4D, 0.2D, 0D);
	private static float NIGHT_SUN_BRIGHTNESS = 0.2F;
	
	/// Return: color of the sun considering half sun is more orange and full sun is more white
	private static Vec3 getSunColor()
	{
		final float SUN_COLOR_CHANGE_LIGHT_LEVEL = 0.6F;
		final Vec3 HALFSUN_COLOR = new Vec3(1.0, 0.6, 0);	// Sun color at sunset/sunrise
		final Vec3 FULLSUN_COLOR = new Vec3(1.0, 1.0, 1.0);	// Sun color at noon

		WorldClient theWorld = Minecraft.getMinecraft().theWorld;
		float skyLight = theWorld.getSunBrightness(1.0F);
		if (skyLight > SUN_COLOR_CHANGE_LIGHT_LEVEL) {
			return FULLSUN_COLOR;
		}
		double r = FULLSUN_COLOR.xCoord*(skyLight-NIGHT_SUN_BRIGHTNESS) + HALFSUN_COLOR.xCoord*(SUN_COLOR_CHANGE_LIGHT_LEVEL-skyLight);
		double g = FULLSUN_COLOR.yCoord*(skyLight-NIGHT_SUN_BRIGHTNESS) + HALFSUN_COLOR.yCoord*(SUN_COLOR_CHANGE_LIGHT_LEVEL-skyLight);
		double b = FULLSUN_COLOR.zCoord*(skyLight-NIGHT_SUN_BRIGHTNESS) + HALFSUN_COLOR.zCoord*(SUN_COLOR_CHANGE_LIGHT_LEVEL-skyLight);
		return new Vec3(r,g,b);
	}
	
	/// Return: hsb[h,s,b] each in the range of [0, MAX_UINT32]
	public static float[] getColorHSB(EntityPlayerSP thePlayer)
	{	
		Entity entity;
		if (thePlayer.isRiding()) {
			entity = thePlayer.ridingEntity;
		} else {
			entity = thePlayer;
		}
		
		int posX = (int)Math.floor(entity.posX);
	    int posY = (int)Math.floor(entity.posY);
	    int posZ = (int)Math.floor(entity.posZ);
	    BlockPos pos = new BlockPos(posX, posY, posZ);
	    WorldClient theWorld = Minecraft.getMinecraft().theWorld;
	    Block block = theWorld.getBlockState(pos).getBlock();
	    int mixedBrightness = block.getMixedBrightnessForBlock(theWorld, pos);	    
	    int skyBrightnessFactor = mixedBrightness >> 20;			// 0 - 15
	    int blockBrightnessFactor = (mixedBrightness >> 4) & 0xF;	// 0 - 15
 
		// Compute brightness
	    float skyLight = Math.abs(theWorld.getSunBrightness(1.0F) - NIGHT_SUN_BRIGHTNESS);
	    float blockLight = theWorld.getLightBrightness(pos);
	    float brightness = (blockBrightnessFactor * blockLight + skyBrightnessFactor * skyLight) * BRIGHTNESS_SCALE / 30;
	    if (brightness <= ZERO_BRIGHTNESS_THRESHOLD) {
	    	return new float[]{0.0F, 0.0F, 0.0F};
	    }
 
	    // Compute color
		Vec3 v = getSunColor();
		int r = (int)((v.xCoord*skyBrightnessFactor + TORCH_COLOR.xCoord*blockBrightnessFactor)/(skyBrightnessFactor+blockBrightnessFactor)*255);
		int g = (int)((v.yCoord*skyBrightnessFactor + TORCH_COLOR.yCoord*blockBrightnessFactor)/(skyBrightnessFactor+blockBrightnessFactor)*255);
		int b = (int)((v.zCoord*skyBrightnessFactor + TORCH_COLOR.zCoord*blockBrightnessFactor)/(skyBrightnessFactor+blockBrightnessFactor)*255);
		r *= brightness;
		g *= brightness;
		b *= brightness;
		float hsb[] = Color.RGBtoHSB(r, g, b, null);
		for (int n=0; n<hsb.length; n++) {
			hsb[n] *= MAX_UINT32;
		}
		
		return hsb;
	}		
}
